#include <stdio.h>
#include <sys/time.h>
#include <math.h>

typedef struct {
  int lower;
  int upper;
} tempevent_type;

#ifdef TESTING
static int numoftrials = 1; /*300*/;
#else
static int numoftrials = 100; /*10*/ /*300*/;
#endif
#define SEED 0x2F678324
#define PERIOD_OF_INDETERMINACY 24
/*
#define PERIOD_OF_INDETERMINACY (24*60*60)
*/
/*
#define PERIOD_OF_INDETERMINACY 2000000
*/
#define NUM_OF_GAMMAS 50
static int gammas[NUM_OF_GAMMAS] = {
1,2,4,6,8,
10,12,14,16,18,
20,22,24,26,28,
30,32,34,36,38,
40,42,44,46,48,
50,52,54,56,58,
60,62,64,66,68,
70,72,74,76,78,
80,82,84,86,88,
90,92,94,96,98};
#define NUM_OF_EVENTS 10
#define NUM_OF_WIDTHS 50
#define MAX_WIDTH 50
static int widths[NUM_OF_WIDTHS];
static tempevent_type events[NUM_OF_EVENTS];
static timings[NUM_OF_GAMMAS][NUM_OF_WIDTHS];

#define PRECISION 256
#define MULT_TABLE_MAX 16  /*if > 16, must change table*/
#define COARSENESS 0x8000
#define COARSENESS_SHIFTED 0x80000000

unsigned char mult_table[MULT_TABLE_MAX][MULT_TABLE_MAX];

typedef struct {
  int left[PRECISION+2];
  int right[PRECISION+2];
} tempdist_type;

typedef enum {FALSE, TRUE} bool;

tempdist_type uniform;
unsigned int uniform_tree_height = 0;

int maxmaxqueue = 0;
int maxqueue = 0;

int pregammas[101];

typedef struct {
  int value;
  unsigned int leftrods;
  unsigned int rightrods;
  void *left;
  void *right;
} tree_node_type;

tree_node_type spare_nodes[10000];
int allocated = 0;
tree_node_type *uniform_tree;

typedef struct {
  unsigned int aprecisionlower;
  unsigned int aprecisionupper;
  unsigned int arightremoved;
  unsigned int aleftremoved;
  unsigned int brightremoved;
  unsigned int bleftremoved;
  tree_node_type *arodstree;
  unsigned int agranulesmid;
  unsigned int agranulesrange;
} queue_node_type;

queue_node_type queue[PRECISION*8 + 2];

tree_node_type *build_spike_tree(unsigned int height) {
  tree_node_type *node, *finger, *build_uniform_tree();

  node = &spare_nodes[allocated++];
  
  if (height < 8)  {
    node->value = 0;
    node->left = NULL;
    node->leftrods = 0;
    node->rightrods = PRECISION;
    node->right = build_spike_tree(height + 1);
    }
  else {
    node->value = 0;
    node->left = NULL;
    node->right = build_uniform_tree(PRECISION,0,0,PRECISION);
    }
  return node;
}

tree_node_type *build_uniform_tree(unsigned int current, unsigned int height,
				   unsigned int lower, unsigned int upper) {
  tree_node_type *node, *finger;

  node = &spare_nodes[allocated++];
  node->value = lower + ((upper - lower) >> 1);
  if (current == 1) {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value - 1;
    /*
    if (upper == PRECISION) node->value = PRECISION;
    if (upper == PRECISION) node->leftrods = PRECISION;
    if (upper == PRECISION) node->rightrods = 0;
    */
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = NULL;
    node->right = NULL;
    }
  else {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value;
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = (void *)build_uniform_tree(current >> 1,height+1,lower,node->value);
    node->right = (void *)build_uniform_tree(current >> 1,height+1,node->value,upper);
    }
  if (height > uniform_tree_height) uniform_tree_height = height;
  /*
  node->height = height;
  */
  return node;
}

int main() {
  int i, j, k;
  bool before(), eval;
  tree_node_type *build_uniform_tree(), *build_spike_tree();
  struct timeval tp1, tp2;
  int temp1, temp2;
  int max_start;
  int ev1, ev2;
  double f;

  for (i=0;i<MULT_TABLE_MAX;i++) for (j=0;j<MULT_TABLE_MAX;j++) mult_table[i][j] = i*j;

#define GAMMA_MULT PRECISION * PRECISION / 100.0
  for (i=0;i<101;i++) {
    f = i;
    pregammas[i] = (int) ceil(f * GAMMA_MULT);
    /* printf("%d %d %f \n", i, pregammas[i], (i * (GAMMA_MULT)));*/
    }

  uniform_tree = build_uniform_tree(PRECISION,0,0,PRECISION);
  /*
  uniform_tree = build_spike_tree(1);
  */
  srand(SEED);

  for (i=0;i<NUM_OF_EVENTS;i++) {
    events[i].lower = 0;
    events[i].upper = 0;
    }
    
  for (i=0;i<NUM_OF_WIDTHS;i++) {
    widths[i] = ((i*(MAX_WIDTH/NUM_OF_WIDTHS)) * PERIOD_OF_INDETERMINACY) + 1;
    }

  events[1].lower = 1;
  events[1].upper = 200;
  events[2].lower = 601;
  events[2].upper = 800;

  for (k=0;k<50;k++) {

    for (i=1;i<=100;i++) {

      gettimeofday(&tp1,NULL);

      for (j=0; j<numoftrials; j++) eval = before(i, &events[1], &events[2]);

      gettimeofday(&tp2,NULL);

      temp1 = tp1.tv_sec * 1000000 + tp1.tv_usec;
      temp2 = tp2.tv_sec * 1000000 + tp2.tv_usec;
      temp1 = temp2 - temp1;

#ifdef TESTING
      if (eval) printf("TRUE %d %d ", k, i);
      else printf("False %d %d ", k, i);
#endif
      printf("%d\n", (temp1/numoftrials));

      }

    printf("\n");
    events[1].lower += 25;
    events[1].upper += 25;
    }

  return TRUE;

}

#include "shared.c"
