/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/ucs/comparison.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef ucs_comparison
#define ucs_comparison

/* comparison.h
*
*  This module implements UCS operations supporting 
*  span comparison operations.  Routines in this module 
*  are called by the query processing system when
*  comparing span time-stamps.
*
*/

/*  
*  Routine:  ucs_s_eq_s
*
*  Description: This routine compares two span values.  It returns
*		the boolean value TRUE if they are equal and 
*		FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: bool : TRUE if span1 = span2
*		        FALSE if span1 <> span2
*
*  Errors:  None
*  
*  Side Effects: None
*/

bool ucs_s_eq_s(span_type span1, span_type span2);

/*  
*  Routine:  ucs_s_lt_s
*
*  Description: This routine compares two span values.  It returns
*		the boolean value TRUE if the first span is less than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: bool  : TRUE if span1 < span2
*            		 FALSE if span1 >= span2
*
*  Errors: None  
*  
*  Side Effects: None
*/

bool ucs_s_lt_s(span_type span1, span_type span2);

/*  
*  Routine:  ucs_s_gt_s
*
*  Description: This routine compares two span values.  It returns
*		the boolean value TRUE if the first span is greater than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value:  bool  : TRUE if span1 > span2
*		          FALSE if span1 <= span2
*
*  Errors: None  
*    
*  Side Effects: None
*/

bool ucs_s_gt_s(span_type span1, span_type span2);
#endif
