/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/test/nowrelative_test.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
/* Add a print statement to tadt_malloc */
#include <stdio.h>
#include <sys/time.h>
#include "system_wide.h"
#include "tadt.h"

static int numoftrials = 100000;
inline static test_add(event_type event, span_type temp, event_type result) {
  void gettimeofday();
  struct timeval tp1, tp2;
  int j, temp1, temp2;

  gettimeofday(&tp1,NULL);
  for (j=0; j<numoftrials; j++) tadt_e_add_fs(event, temp, result);
  gettimeofday(&tp2,NULL);

  temp1 = tp1.tv_sec * 1000000 + tp1.tv_usec;
  temp2 = tp2.tv_sec * 1000000 + tp2.tv_usec;
  temp1 = temp2 - temp1;
  printf("in micro seconds %d\n", (temp1/numoftrials));
}

inline static test_unpack(event_type event, event_type temp) {
  void gettimeofday();
  struct timeval tp1, tp2;
  int j, temp1, temp2;

  gettimeofday(&tp1,NULL);
  for (j=0; j<numoftrials; j++) tadt_unpack_e(event, temp);
  gettimeofday(&tp2,NULL);

  temp1 = tp1.tv_sec * 1000000 + tp1.tv_usec;
  temp2 = tp2.tv_sec * 1000000 + tp2.tv_usec;
  temp1 = temp2 - temp1;
  printf("in micro seconds %d\n", (temp1/numoftrials));
}

void main()
{
  interval_type interval;
  span_type span;
  event_type present;
  event_type temp;
  event_type event, event1;
  void printf();

  error_handler();
  tadt_init();
  temp = tadt_allocate_e(upack);

  event = tadt_allocate_e(now_relative);
  event->nowrelative.sign = 1;
  event->nowrelative.chunksize = 5;
  event->nowrelative.chunks = 120;
  event->nowrelative.secondshi = 0xFFFF;
  event->nowrelative.secondslow = 0xFFF;
  printf("nowrelative ind "); test_unpack(event, temp);

  event = tadt_allocate_e(special);
  event->hires.sign = 1;
  event->special.nowrelative = 1;
  event->hires.secondshi = 0xFFFF;
  event->hires.secondslow = 0xFFF;
  event->hires.milli = 2;
  event->hires.micro = 2;
  printf("nowrelative "); test_unpack(event, temp);

  event = tadt_allocate_e(hi);
  event->hires.sign = 0;
  event->hires.secondshi = 0xFFFF;
  event->hires.secondslow = 0xFFF;
  event->hires.milli = 2;
  event->hires.micro = 2;
  printf("hi "); test_unpack(event, temp);

  event = tadt_allocate_e(low);
  event->lowres.sign = 0;
  event->lowres.secondshi = 0xFFFF;
  event->lowres.secondslow = 0xFFF;
  printf("low "); test_unpack(event, temp);

  event = tadt_allocate_e(extended);
  event->exres.sign = 0;
  event->exres.secondshi = 0xFFFF;
  event->exres.secondslow = 0xFFF;
  event->exres.milli = 2;
  event->exres.micro = 2;
  event->exres.nano = 2;
  printf("extended "); test_unpack(event, temp);

  span = tadt_allocate_s(upack);
  event1 = tadt_allocate_s(upack);
  tadt_copy_e(temp, (span_type) span);
  printf("add 2 det"); test_add(temp, span, event1);

  event = tadt_allocate_e(hi_chunked);
  event->hi_chunked.sign = 0;
  event->hi_chunked.secondshi = 0xFFFF;
  event->hi_chunked.secondslow = 0xFFF;
  event->hi_chunked.chunksize = 5;
  event->hi_chunked.chunks = 100;
  event->hi_chunked.milli = 2;
  printf("hi_chunked "); test_unpack(event, temp);

  event = tadt_allocate_e(hi_default_chunked);
  event->hi_default_chunked.sign = 0;
  event->hi_default_chunked.secondshi = 0xFFFF;
  event->hi_default_chunked.secondslow = 0xFFF;
  event->hi_default_chunked.chunksize = 5;
  event->hi_default_chunked.chunks = 100;
  event->hi_default_chunked.milli = 2;
  printf("hi_default_chunked "); test_unpack(event, temp);

  span = tadt_allocate_s(upack);
  event1 = tadt_allocate_s(upack);
  tadt_copy_e(temp, (span_type) span);
  printf("add 2 ind "); test_add(temp, span, event1);

/*
  dump_event(span,"");
  dump_event(temp,"");
*/
/*
  dump_event(temp,"");
*/

  /*
  event = tadt_allocate_e(hi_chunked);
  event = tadt_allocate_e(hi);
  temp = tadt_allocate_e(upack);
  present = tadt_allocate_e(upack);
  span = tadt_allocate_s(upack);

  span->upack.indeterminate = FALSE;
  temp->upack.indeterminate = FALSE;
  present->upack.indeterminate = FALSE;

  span->upack.sign = POS;
  span->upack.seconds = 1;
  event->hires.secondshi = 0xFFFFFFFF;
  event->hires.secondslow = 0xFFFFFFFF;
  event->hires.sign = NEG;
  event->special.ts_type = special;
  event->special.nowrelative = TRUE;
  tadt_present_as_of_query(FALSE, upack, present);
  tadt_unpack_e(event,event);
  dump_event((event_type) span,"right");
  dump_event(event,"");
  dump_event(present,"");
  printf("here\n");
  dump_event(event,"");
  tadt_e_add_fs(event, span, temp);
  printf("after\n");
  dump_event(event,"");
  dump_event(temp,"");
  tadt_e_minus_fs(present, span, present);
  dump_event((event_type) span,"");
  dump_event(event,"");
  dump_event(present,"");
  */
}
